namespace Parsing.Data;

public static class DataManipulationHelpers
{
    public static TType ReduceData<TType>(this List<TType> data, Func<TType, TType, TType> reducer)
    {
        if (data.Count < 2)
        {
            return data[0];
        }
        TType result = data[0];
        for (int i = 1; i < data.Count; i++)
        {
            result = reducer(result, data[i]);
        }
        return result;
    }

    public static TType ReduceData<TType>(this List<TType> data, Func<List<TType>, TType> reducer)
    {
        return reducer(data);
    }

    public static List<TNewType> TransformData<TType, TNewType>(this List<TType> data, Func<TType, TNewType> transformer)
    {
        var newList = new List<TNewType>();
        foreach (TType dataItem in data)
        {
            newList.Add(transformer(dataItem));
        }
        return newList;
    }

    public static List<TNewType> TransformData<TType, TNewType>(this List<List<TType>> data, Func<List<TType>, TNewType> transformer)
    {
        var newList = new List<TNewType>();
        foreach (List<TType> dataItemList in data)
        {
            newList.Add(transformer(dataItemList));
        }
        return newList;
    }
}