using System.Runtime.InteropServices;
using Parsing.Data;

namespace Parsing.Data;

public static class DefaultTwoDimensionalManipulator
{
    public static DefaultTwoDimensionalManipulator<TDataType> Create<TDataType>(List<List<TDataType>> dataSet) where TDataType : IEquatable<TDataType>
    {
        return new DefaultTwoDimensionalManipulator<TDataType>(dataSet);
    }
}

public class DefaultTwoDimensionalManipulator<TDataType> : DataSetManipulatorBase<List<TDataType>, TDataType, int> where TDataType : IEquatable<TDataType>
{
    public DefaultTwoDimensionalManipulator(List<List<TDataType>> dataSet) : base(dataSet, new DefaultDataSetIndexer<TDataType>())
    {
    }

    protected override Direction ValidDirections()
    {
        return (Direction.N
                | Direction.NE
                | Direction.E
                | Direction.SE
                | Direction.S
                | Direction.SW
                | Direction.W
                | Direction.NW);
    }

    public override bool IsValidIndex(IDataIndex<int> queryPosition)
    {
        var xIndex = queryPosition.GetIndices()[0];
        var yIndex = queryPosition.GetIndices()[1];
        return (yIndex >= 0) && (yIndex < this.dataSet.Count) && (xIndex >= 0) && (xIndex < this.dataSet[yIndex].Count);
    }

    public override IDataIndex<int> Move(IDataIndex<int> currentPosition, Direction direction)
    {
        switch (direction)
        {
            case Direction.N:
                return new DefaultPositionalDataIndex(currentPosition.GetIndices()[0], currentPosition.GetIndices()[1] + 1);
            case Direction.NE:
                return new DefaultPositionalDataIndex(currentPosition.GetIndices()[0] + 1, currentPosition.GetIndices()[1] + 1);
            case Direction.E:
                return new DefaultPositionalDataIndex(currentPosition.GetIndices()[0] + 1, currentPosition.GetIndices()[1]);
            case Direction.SE:
                return new DefaultPositionalDataIndex(currentPosition.GetIndices()[0] + 1, currentPosition.GetIndices()[1] - 1);
            case Direction.S:
                return new DefaultPositionalDataIndex(currentPosition.GetIndices()[0], currentPosition.GetIndices()[1] - 1);
            case Direction.SW:
                return new DefaultPositionalDataIndex(currentPosition.GetIndices()[0] - 1, currentPosition.GetIndices()[1] - 1);
            case Direction.W:
                return new DefaultPositionalDataIndex(currentPosition.GetIndices()[0] - 1, currentPosition.GetIndices()[1]);
            case Direction.NW:
                return new DefaultPositionalDataIndex(currentPosition.GetIndices()[0] - 1, currentPosition.GetIndices()[1] + 1);
            default:
                throw new ArgumentOutOfRangeException("Direction was not accounted for move for current data set!");
        }
    }

    public override List<SearchResult<int>> FindInSet(TDataType data)
    {
        var results = new List<SearchResult<int>>();

        for (int y = 0; y < this.dataSet.Count; y++)
        {
            for (int x = 0; x < this.dataSet[this.dataSet.Count - y - 1].Count; x++)
            {
                if (EqualityComparer<TDataType>.Default.Equals(this.dataSet[this.dataSet.Count - y - 1][x], data))
                {
                    var singleResult = new SearchResult<int>(new DefaultPositionalDataIndex(x, y));
                    results.Add(singleResult);
                }
            }
        }

        return results;
    }
}