﻿namespace Parsing;

using System;
using System.Collections.Generic;
using Parsing.Data;
using Parsing.Schema;
using Parsing.Tokenization;

public static class TextParser
{
    public static TextParser<TContext> Create<TContext>(ISchema<TContext> schema, string[]? delimiters = null, bool removeEmptyEntries = true) where TContext : ISchemaContext
    {
        return new TextParser<TContext>(schema, delimiters, removeEmptyEntries);
    }
}

public class TextParser<T> : TokenConverter where T : ISchemaContext
{
    private LineParser<T> lineParser;
    private string[] lines;
    private bool removeEmptyEntries;

    public TextParser(ISchema<T> schema, string[]? delimiters = null, bool removeEmptyEntries = true) : base()
    {
        this.lineParser = new LineParser<T>(schema, delimiters, removeEmptyEntries);
        this.lines = new string[] { };
        this.removeEmptyEntries = removeEmptyEntries;
    }

    public TextParser<T> SetInputText(string text)
    {
        var options = StringSplitOptions.TrimEntries;
        if (removeEmptyEntries)
        {
            options = options | StringSplitOptions.RemoveEmptyEntries;
        }
        this.lines = text.Split("\n", options);
        return this;
    }

    public TextParser<T> Parse()
    {
        foreach (var line in this.lines)
        {
            this.rawTokens.Add(this.lineParser.ParseLine(line));
        }
        return this;
    }
}
