﻿namespace Parsing.Tokenization;

public class InputProvider
{
    public class LookaheadContext : IDisposable
    {
        private InputProvider contextedProvider;
        private int initialPosition;

        public LookaheadContext(InputProvider contextedProvider)
        {
            this.contextedProvider = contextedProvider;
            this.initialPosition = contextedProvider.CurrentPosition;
            contextedProvider.IsBeingLookedAhead = true;
        }

        public void Dispose()
        {
            this.contextedProvider.CurrentPosition = this.initialPosition;
            contextedProvider.IsBeingLookedAhead = false;
        }
    }

    private string[] words;
    private bool IsBeingLookedAhead { get; set; } = false;

    private int CurrentPosition { get; set; }

    public InputProvider(string[] words)
    {
        this.words = words;
        this.CurrentPosition = 0;
    }

    public InputProvider(string text)
    {
        this.words = text.Split("\n");
        this.CurrentPosition = 0;
    }

    public InputProvider.LookaheadContext GetLookaheadContext()
    {
        return new InputProvider.LookaheadContext(this);
    }

    public bool CanYieldWord()
    {
        return this.CurrentPosition < this.words.Length;
    }

    public void SkipCurrentWord()
    {
        this.CurrentPosition += 1;
    }

    public string YieldWord()
    {
        if (!this.CanYieldWord())
        {
            return string.Empty;
        }

        var wordToProcess = this.words[this.CurrentPosition];
        this.CurrentPosition++;
        return wordToProcess;
    }
}
